@@ -1,5 +1,8 @@
-//! Persistent storage for accounts. For more information, see:
-//! https://docs.solana.com/implemented-proposals/persistent-account-storage
+//! Persistent storage for accounts.
+//!
+//! For more information, see:
+//!
+//! <https://docs.solana.com/implemented-proposals/persistent-account-storage>
 
 use {
     log::*,
@@ -13,6 +16,7 @@ use {
     },
     std::{
         borrow::Borrow,
+        convert::TryFrom,
         fs::{remove_file, OpenOptions},
         io::{self, Seek, SeekFrom, Write},
         mem,
@@ -33,7 +37,7 @@ macro_rules! u64_align {
     };
 }
 
-const MAXIMUM_APPEND_VEC_FILE_SIZE: usize = 16 * 1024 * 1024 * 1024; // 16 GiB
+const MAXIMUM_APPEND_VEC_FILE_SIZE: u64 = 16 * 1024 * 1024 * 1024; // 16 GiB
 
 pub type StoredMetaWriteVersion = u64;
 
@@ -256,7 +260,10 @@ impl AppendVec {
                 std::io::ErrorKind::Other,
                 format!("too small file size {} for AppendVec", file_size),
             ))
-        } else if file_size > MAXIMUM_APPEND_VEC_FILE_SIZE {
+        } else if usize::try_from(MAXIMUM_APPEND_VEC_FILE_SIZE)
+            .map(|max| file_size > max)
+            .unwrap_or(true)
+        {
             Err(std::io::Error::new(
                 std::io::ErrorKind::Other,
                 format!("too large file size {} for AppendVec", file_size),
