@@ -16,6 +16,9 @@ pub enum CostTrackerError {
     /// would exceed block max limit
     WouldExceedBlockMaxLimit,
 
+    /// would exceed vote max limit
+    WouldExceedVoteMaxLimit,
+
     /// would exceed account max limit
     WouldExceedAccountMaxLimit,
 }
@@ -24,59 +27,70 @@ pub enum CostTrackerError {
 pub struct CostTracker {
     account_cost_limit: u64,
     block_cost_limit: u64,
+    vote_cost_limit: u64,
     cost_by_writable_accounts: HashMap<Pubkey, u64>,
     block_cost: u64,
+    vote_cost: u64,
     transaction_count: u64,
 }
 
 impl Default for CostTracker {
     fn default() -> Self {
-        CostTracker::new(MAX_WRITABLE_ACCOUNT_UNITS, MAX_BLOCK_UNITS)
+        CostTracker::new(MAX_WRITABLE_ACCOUNT_UNITS, MAX_BLOCK_UNITS, MAX_VOTE_UNITS)
     }
 }
 
 impl CostTracker {
-    pub fn new(account_cost_limit: u64, block_cost_limit: u64) -> Self {
+    pub fn new(account_cost_limit: u64, block_cost_limit: u64, vote_cost_limit: u64) -> Self {
         assert!(account_cost_limit <= block_cost_limit);
+        assert!(vote_cost_limit <= block_cost_limit);
         Self {
             account_cost_limit,
             block_cost_limit,
+            vote_cost_limit,
             cost_by_writable_accounts: HashMap::with_capacity(WRITABLE_ACCOUNTS_PER_BLOCK),
             block_cost: 0,
+            vote_cost: 0,
             transaction_count: 0,
         }
     }
 
     // bench tests needs to reset limits
-    pub fn set_limits(&mut self, account_cost_limit: u64, block_cost_limit: u64) {
+    pub fn set_limits(
+        &mut self,
+        account_cost_limit: u64,
+        block_cost_limit: u64,
+        vote_cost_limit: u64,
+    ) {
         self.account_cost_limit = account_cost_limit;
         self.block_cost_limit = block_cost_limit;
+        self.vote_cost_limit = vote_cost_limit;
     }
 
     pub fn would_transaction_fit(
         &self,
-        _transaction: &SanitizedTransaction,
+        transaction: &SanitizedTransaction,
         tx_cost: &TransactionCost,
     ) -> Result<(), CostTrackerError> {
-        self.would_fit(&tx_cost.writable_accounts, &tx_cost.sum())
+        self.would_fit(&tx_cost.writable_accounts, tx_cost.sum(), transaction)
     }
 
     pub fn add_transaction_cost(
         &mut self,
-        _transaction: &SanitizedTransaction,
+        transaction: &SanitizedTransaction,
         tx_cost: &TransactionCost,
     ) {
-        self.add_transaction(&tx_cost.writable_accounts, &tx_cost.sum());
+        self.add_transaction(&tx_cost.writable_accounts, tx_cost.sum(), transaction);
     }
 
     pub fn try_add(
         &mut self,
-        _transaction: &SanitizedTransaction,
+        transaction: &SanitizedTransaction,
         tx_cost: &TransactionCost,
     ) -> Result<u64, CostTrackerError> {
-        let cost = tx_cost.sum() * tx_cost.cost_weight as u64;
-        self.would_fit(&tx_cost.writable_accounts, &cost)?;
-        self.add_transaction(&tx_cost.writable_accounts, &cost);
+        let cost = tx_cost.sum();
+        self.would_fit(&tx_cost.writable_accounts, cost, transaction)?;
+        self.add_transaction(&tx_cost.writable_accounts, cost, transaction);
         Ok(self.block_cost)
     }
 
@@ -92,6 +106,7 @@ impl CostTracker {
             "cost_tracker_stats",
             ("bank_slot", bank_slot as i64, i64),
             ("block_cost", self.block_cost as i64, i64),
+            ("vote_cost", self.vote_cost as i64, i64),
             ("transaction_count", self.transaction_count as i64, i64),
             (
                 "number_of_accounts",
@@ -116,14 +131,26 @@ impl CostTracker {
         (costliest_account, costliest_account_cost)
     }
 
-    fn would_fit(&self, keys: &[Pubkey], cost: &u64) -> Result<(), CostTrackerError> {
+    fn would_fit(
+        &self,
+        keys: &[Pubkey],
+        cost: u64,
+        transaction: &SanitizedTransaction,
+    ) -> Result<(), CostTrackerError> {
         // check against the total package cost
-        if self.block_cost + cost > self.block_cost_limit {
+        if self.block_cost.saturating_add(cost) > self.block_cost_limit {
             return Err(CostTrackerError::WouldExceedBlockMaxLimit);
         }
 
+        // if vote transaction, check if it exceeds vote_transaction_limit
+        if transaction.is_simple_vote_transaction()
+            && self.vote_cost.saturating_add(cost) > self.vote_cost_limit
+        {
+            return Err(CostTrackerError::WouldExceedVoteMaxLimit);
+        }
+
         // check if the transaction itself is more costly than the account_cost_limit
-        if *cost > self.account_cost_limit {
+        if cost > self.account_cost_limit {
             return Err(CostTrackerError::WouldExceedAccountMaxLimit);
         }
 
@@ -131,7 +158,7 @@ impl CostTracker {
         for account_key in keys.iter() {
             match self.cost_by_writable_accounts.get(account_key) {
                 Some(chained_cost) => {
-                    if chained_cost + cost > self.account_cost_limit {
+                    if chained_cost.saturating_add(cost) > self.account_cost_limit {
                         return Err(CostTrackerError::WouldExceedAccountMaxLimit);
                     } else {
                         continue;
@@ -144,15 +171,19 @@ impl CostTracker {
         Ok(())
     }
 
-    fn add_transaction(&mut self, keys: &[Pubkey], cost: &u64) {
+    fn add_transaction(&mut self, keys: &[Pubkey], cost: u64, transaction: &SanitizedTransaction) {
         for account_key in keys.iter() {
-            *self
+            let account_cost = self
                 .cost_by_writable_accounts
                 .entry(*account_key)
-                .or_insert(0) += cost;
+                .or_insert(0);
+            *account_cost = account_cost.saturating_add(cost);
         }
-        self.block_cost += cost;
-        self.transaction_count += 1;
+        self.block_cost = self.block_cost.saturating_add(cost);
+        if transaction.is_simple_vote_transaction() {
+            self.vote_cost = self.vote_cost.saturating_add(cost);
+        }
+        self.transaction_count = self.transaction_count.saturating_add(1);
     }
 }
 
@@ -168,8 +199,9 @@ mod tests {
             hash::Hash,
             signature::{Keypair, Signer},
             system_transaction,
-            transaction::Transaction,
+            transaction::{TransactionError, VersionedTransaction},
         },
+        solana_vote_program::vote_transaction,
         std::{cmp, sync::Arc},
     };
 
@@ -188,19 +220,46 @@ mod tests {
     fn build_simple_transaction(
         mint_keypair: &Keypair,
         start_hash: &Hash,
-    ) -> (Transaction, Vec<Pubkey>, u64) {
+    ) -> (SanitizedTransaction, Vec<Pubkey>, u64) {
         let keypair = Keypair::new();
-        let simple_transaction =
-            system_transaction::transfer(mint_keypair, &keypair.pubkey(), 2, *start_hash);
+        let simple_transaction = SanitizedTransaction::from_transaction_for_tests(
+            system_transaction::transfer(mint_keypair, &keypair.pubkey(), 2, *start_hash),
+        );
 
         (simple_transaction, vec![mint_keypair.pubkey()], 5)
     }
 
+    fn build_simple_vote_transaction(
+        mint_keypair: &Keypair,
+        start_hash: &Hash,
+    ) -> (SanitizedTransaction, Vec<Pubkey>, u64) {
+        let keypair = Keypair::new();
+        let transaction = vote_transaction::new_vote_transaction(
+            vec![42],
+            Hash::default(),
+            *start_hash,
+            mint_keypair,
+            &keypair,
+            &keypair,
+            None,
+        );
+        let message_hash = transaction.message.hash();
+        let vote_transaction = SanitizedTransaction::try_create(
+            VersionedTransaction::from(transaction),
+            message_hash,
+            Some(true),
+            |_| Err(TransactionError::UnsupportedVersion),
+        )
+        .unwrap();
+        (vote_transaction, vec![mint_keypair.pubkey()], 10)
+    }
+
     #[test]
     fn test_cost_tracker_initialization() {
-        let testee = CostTracker::new(10, 11);
+        let testee = CostTracker::new(10, 11, 8);
         assert_eq!(10, testee.account_cost_limit);
         assert_eq!(11, testee.block_cost_limit);
+        assert_eq!(8, testee.vote_cost_limit);
         assert_eq!(0, testee.cost_by_writable_accounts.len());
         assert_eq!(0, testee.block_cost);
     }
@@ -208,75 +267,97 @@ mod tests {
     #[test]
     fn test_cost_tracker_ok_add_one() {
         let (mint_keypair, start_hash) = test_setup();
-        let (_tx, keys, cost) = build_simple_transaction(&mint_keypair, &start_hash);
+        let (tx, keys, cost) = build_simple_transaction(&mint_keypair, &start_hash);
 
         // build testee to have capacity for one simple transaction
-        let mut testee = CostTracker::new(cost, cost);
-        assert!(testee.would_fit(&keys, &cost).is_ok());
-        testee.add_transaction(&keys, &cost);
+        let mut testee = CostTracker::new(cost, cost, cost);
+        assert!(testee.would_fit(&keys, cost, &tx).is_ok());
+        testee.add_transaction(&keys, cost, &tx);
         assert_eq!(cost, testee.block_cost);
+        assert_eq!(0, testee.vote_cost);
+        let (_costliest_account, costliest_account_cost) = testee.find_costliest_account();
+        assert_eq!(cost, costliest_account_cost);
+    }
+
+    #[test]
+    fn test_cost_tracker_ok_add_one_vote() {
+        let (mint_keypair, start_hash) = test_setup();
+        let (tx, keys, cost) = build_simple_vote_transaction(&mint_keypair, &start_hash);
+
+        // build testee to have capacity for one simple transaction
+        let mut testee = CostTracker::new(cost, cost, cost);
+        assert!(testee.would_fit(&keys, cost, &tx).is_ok());
+        testee.add_transaction(&keys, cost, &tx);
+        assert_eq!(cost, testee.block_cost);
+        assert_eq!(cost, testee.vote_cost);
+        let (_ccostliest_account, costliest_account_cost) = testee.find_costliest_account();
+        assert_eq!(cost, costliest_account_cost);
     }
 
     #[test]
     fn test_cost_tracker_ok_add_two_same_accounts() {
         let (mint_keypair, start_hash) = test_setup();
         // build two transactions with same signed account
-        let (_tx1, keys1, cost1) = build_simple_transaction(&mint_keypair, &start_hash);
-        let (_tx2, keys2, cost2) = build_simple_transaction(&mint_keypair, &start_hash);
+        let (tx1, keys1, cost1) = build_simple_transaction(&mint_keypair, &start_hash);
+        let (tx2, keys2, cost2) = build_simple_transaction(&mint_keypair, &start_hash);
 
         // build testee to have capacity for two simple transactions, with same accounts
-        let mut testee = CostTracker::new(cost1 + cost2, cost1 + cost2);
+        let mut testee = CostTracker::new(cost1 + cost2, cost1 + cost2, cost1 + cost2);
         {
-            assert!(testee.would_fit(&keys1, &cost1).is_ok());
-            testee.add_transaction(&keys1, &cost1);
+            assert!(testee.would_fit(&keys1, cost1, &tx1).is_ok());
+            testee.add_transaction(&keys1, cost1, &tx1);
         }
         {
-            assert!(testee.would_fit(&keys2, &cost2).is_ok());
-            testee.add_transaction(&keys2, &cost2);
+            assert!(testee.would_fit(&keys2, cost2, &tx2).is_ok());
+            testee.add_transaction(&keys2, cost2, &tx2);
         }
         assert_eq!(cost1 + cost2, testee.block_cost);
         assert_eq!(1, testee.cost_by_writable_accounts.len());
+        let (_ccostliest_account, costliest_account_cost) = testee.find_costliest_account();
+        assert_eq!(cost1 + cost2, costliest_account_cost);
     }
 
     #[test]
     fn test_cost_tracker_ok_add_two_diff_accounts() {
         let (mint_keypair, start_hash) = test_setup();
         // build two transactions with diff accounts
-        let (_tx1, keys1, cost1) = build_simple_transaction(&mint_keypair, &start_hash);
+        let (tx1, keys1, cost1) = build_simple_transaction(&mint_keypair, &start_hash);
         let second_account = Keypair::new();
-        let (_tx2, keys2, cost2) = build_simple_transaction(&second_account, &start_hash);
+        let (tx2, keys2, cost2) = build_simple_transaction(&second_account, &start_hash);
 
         // build testee to have capacity for two simple transactions, with same accounts
-        let mut testee = CostTracker::new(cmp::max(cost1, cost2), cost1 + cost2);
+        let mut testee = CostTracker::new(cmp::max(cost1, cost2), cost1 + cost2, cost1 + cost2);
         {
-            assert!(testee.would_fit(&keys1, &cost1).is_ok());
-            testee.add_transaction(&keys1, &cost1);
+            assert!(testee.would_fit(&keys1, cost1, &tx1).is_ok());
+            testee.add_transaction(&keys1, cost1, &tx1);
         }
         {
-            assert!(testee.would_fit(&keys2, &cost2).is_ok());
-            testee.add_transaction(&keys2, &cost2);
+            assert!(testee.would_fit(&keys2, cost2, &tx2).is_ok());
+            testee.add_transaction(&keys2, cost2, &tx2);
         }
         assert_eq!(cost1 + cost2, testee.block_cost);
         assert_eq!(2, testee.cost_by_writable_accounts.len());
+        let (_ccostliest_account, costliest_account_cost) = testee.find_costliest_account();
+        assert_eq!(std::cmp::max(cost1, cost2), costliest_account_cost);
     }
 
     #[test]
     fn test_cost_tracker_chain_reach_limit() {
         let (mint_keypair, start_hash) = test_setup();
         // build two transactions with same signed account
-        let (_tx1, keys1, cost1) = build_simple_transaction(&mint_keypair, &start_hash);
-        let (_tx2, keys2, cost2) = build_simple_transaction(&mint_keypair, &start_hash);
+        let (tx1, keys1, cost1) = build_simple_transaction(&mint_keypair, &start_hash);
+        let (tx2, keys2, cost2) = build_simple_transaction(&mint_keypair, &start_hash);
 
         // build testee to have capacity for two simple transactions, but not for same accounts
-        let mut testee = CostTracker::new(cmp::min(cost1, cost2), cost1 + cost2);
+        let mut testee = CostTracker::new(cmp::min(cost1, cost2), cost1 + cost2, cost1 + cost2);
         // should have room for first transaction
         {
-            assert!(testee.would_fit(&keys1, &cost1).is_ok());
-            testee.add_transaction(&keys1, &cost1);
+            assert!(testee.would_fit(&keys1, cost1, &tx1).is_ok());
+            testee.add_transaction(&keys1, cost1, &tx1);
         }
         // but no more sapce on the same chain (same signer account)
         {
-            assert!(testee.would_fit(&keys2, &cost2).is_err());
+            assert!(testee.would_fit(&keys2, cost2, &tx2).is_err());
         }
     }
 
@@ -284,28 +365,56 @@ mod tests {
     fn test_cost_tracker_reach_limit() {
         let (mint_keypair, start_hash) = test_setup();
         // build two transactions with diff accounts
-        let (_tx1, keys1, cost1) = build_simple_transaction(&mint_keypair, &start_hash);
+        let (tx1, keys1, cost1) = build_simple_transaction(&mint_keypair, &start_hash);
         let second_account = Keypair::new();
-        let (_tx2, keys2, cost2) = build_simple_transaction(&second_account, &start_hash);
+        let (tx2, keys2, cost2) = build_simple_transaction(&second_account, &start_hash);
 
         // build testee to have capacity for each chain, but not enough room for both transactions
-        let mut testee = CostTracker::new(cmp::max(cost1, cost2), cost1 + cost2 - 1);
+        let mut testee =
+            CostTracker::new(cmp::max(cost1, cost2), cost1 + cost2 - 1, cost1 + cost2 - 1);
         // should have room for first transaction
         {
-            assert!(testee.would_fit(&keys1, &cost1).is_ok());
-            testee.add_transaction(&keys1, &cost1);
+            assert!(testee.would_fit(&keys1, cost1, &tx1).is_ok());
+            testee.add_transaction(&keys1, cost1, &tx1);
         }
         // but no more room for package as whole
         {
-            assert!(testee.would_fit(&keys2, &cost2).is_err());
+            assert!(testee.would_fit(&keys2, cost2, &tx2).is_err());
+        }
+    }
+
+    #[test]
+    fn test_cost_tracker_reach_vote_limit() {
+        let (mint_keypair, start_hash) = test_setup();
+        // build two mocking vote transactions with diff accounts
+        let (tx1, keys1, cost1) = build_simple_vote_transaction(&mint_keypair, &start_hash);
+        let second_account = Keypair::new();
+        let (tx2, keys2, cost2) = build_simple_vote_transaction(&second_account, &start_hash);
+
+        // build testee to have capacity for each chain, but not enough room for both votes
+        let mut testee = CostTracker::new(cmp::max(cost1, cost2), cost1 + cost2, cost1 + cost2 - 1);
+        // should have room for first vote
+        {
+            assert!(testee.would_fit(&keys1, cost1, &tx1).is_ok());
+            testee.add_transaction(&keys1, cost1, &tx1);
+        }
+        // but no more room for package as whole
+        {
+            assert!(testee.would_fit(&keys2, cost2, &tx2).is_err());
+        }
+        // however there is room for none-vote tx3
+        {
+            let third_account = Keypair::new();
+            let (tx3, keys3, cost3) = build_simple_transaction(&third_account, &start_hash);
+            assert!(testee.would_fit(&keys3, cost3, &tx3).is_ok());
         }
     }
 
     #[test]
     fn test_cost_tracker_try_add_is_atomic() {
         let (mint_keypair, start_hash) = test_setup();
-        let (tx, _keys, _cost) = build_simple_transaction(&mint_keypair, &start_hash);
-        let tx = SanitizedTransaction::from_transaction_for_tests(tx);
+        // build two mocking vote transactions with diff accounts
+        let (tx1, _keys1, _cost1) = build_simple_vote_transaction(&mint_keypair, &start_hash);
 
         let acct1 = Pubkey::new_unique();
         let acct2 = Pubkey::new_unique();
@@ -314,7 +423,7 @@ mod tests {
         let account_max = cost * 2;
         let block_max = account_max * 3; // for three accts
 
-        let mut testee = CostTracker::new(account_max, block_max);
+        let mut testee = CostTracker::new(account_max, block_max, block_max);
 
         // case 1: a tx writes to 3 accounts, should success, we will have:
         // | acct1 | $cost |
@@ -327,7 +436,7 @@ mod tests {
                 execution_cost: cost,
                 ..TransactionCost::default()
             };
-            assert!(testee.try_add(&tx, &tx_cost).is_ok());
+            assert!(testee.try_add(&tx1, &tx_cost).is_ok());
             let (_costliest_account, costliest_account_cost) = testee.find_costliest_account();
             assert_eq!(cost, testee.block_cost);
             assert_eq!(3, testee.cost_by_writable_accounts.len());
@@ -345,7 +454,7 @@ mod tests {
                 execution_cost: cost,
                 ..TransactionCost::default()
             };
-            assert!(testee.try_add(&tx, &tx_cost).is_ok());
+            assert!(testee.try_add(&tx1, &tx_cost).is_ok());
             let (costliest_account, costliest_account_cost) = testee.find_costliest_account();
             assert_eq!(cost * 2, testee.block_cost);
             assert_eq!(3, testee.cost_by_writable_accounts.len());
@@ -365,7 +474,7 @@ mod tests {
                 execution_cost: cost,
                 ..TransactionCost::default()
             };
-            assert!(testee.try_add(&tx, &tx_cost).is_err());
+            assert!(testee.try_add(&tx1, &tx_cost).is_err());
             let (costliest_account, costliest_account_cost) = testee.find_costliest_account();
             assert_eq!(cost * 2, testee.block_cost);
             assert_eq!(3, testee.cost_by_writable_accounts.len());
@@ -373,26 +482,4 @@ mod tests {
             assert_eq!(acct2, costliest_account);
         }
     }
-
-    #[test]
-    fn test_try_add_with_cost_weight() {
-        let (mint_keypair, start_hash) = test_setup();
-        let (tx, _keys, _cost) = build_simple_transaction(&mint_keypair, &start_hash);
-        let tx = SanitizedTransaction::from_transaction_for_tests(tx);
-
-        let limit = 100u64;
-        let mut testee = CostTracker::new(limit, limit);
-
-        let mut cost = TransactionCost {
-            execution_cost: limit + 1,
-            ..TransactionCost::default()
-        };
-
-        // cost exceed limit by 1, will not fit
-        assert!(testee.try_add(&tx, &cost).is_err());
-
-        cost.cost_weight = 0u32;
-        // setting cost_weight to zero will allow this tx
-        assert!(testee.try_add(&tx, &cost).is_ok());
-    }
 }
