@@ -1,5 +1,6 @@
 #![cfg_attr(RUSTC_WITH_SPECIALIZATION, feature(min_specialization))]
 #![allow(clippy::integer_arithmetic)]
+pub mod account_rent_state;
 pub mod accounts;
 pub mod accounts_background_service;
 pub mod accounts_cache;
@@ -16,7 +17,6 @@ pub mod bank_forks;
 pub mod bank_utils;
 pub mod block_cost_limits;
 pub mod blockhash_queue;
-pub mod bloom;
 pub mod bucket_map_holder;
 pub mod bucket_map_holder_stats;
 pub mod builtins;
@@ -55,7 +55,9 @@ pub mod stakes;
 pub mod status_cache;
 mod system_instruction_processor;
 pub mod transaction_batch;
+pub mod transaction_cost_metrics_sender;
 pub mod vote_account;
+pub mod vote_parser;
 pub mod vote_sender_types;
 pub mod waitable_condvar;
 
