@@ -2,28 +2,28 @@ use {
     serde::{Deserialize, Serialize},
     solana_measure::measure::Measure,
     solana_program_runtime::{
+        compute_budget::ComputeBudget,
         instruction_recorder::InstructionRecorder,
         invoke_context::{
             BuiltinProgram, Executors, InvokeContext, ProcessInstructionResult,
             TransactionAccountRefCell,
         },
         log_collector::LogCollector,
+        sysvar_cache::SysvarCache,
         timings::ExecuteTimings,
     },
     solana_sdk::{
         account::WritableAccount,
-        compute_budget::ComputeBudget,
         feature_set::{prevent_calling_precompiles_as_programs, FeatureSet},
         hash::Hash,
         message::SanitizedMessage,
         precompiles::is_precompile,
-        pubkey::Pubkey,
         rent::Rent,
         saturating_add_assign,
         sysvar::instructions,
         transaction::TransactionError,
     },
-    std::{cell::RefCell, rc::Rc, sync::Arc},
+    std::{borrow::Cow, cell::RefCell, rc::Rc, sync::Arc},
 };
 
 #[derive(Debug, Default, Clone, Deserialize, Serialize)]
@@ -64,7 +64,7 @@ impl MessageProcessor {
         feature_set: Arc<FeatureSet>,
         compute_budget: ComputeBudget,
         timings: &mut ExecuteTimings,
-        sysvars: &[(Pubkey, Vec<u8>)],
+        sysvar_cache: &SysvarCache,
         blockhash: Hash,
         lamports_per_signature: u64,
         current_accounts_data_len: u64,
@@ -73,7 +73,7 @@ impl MessageProcessor {
             rent,
             accounts,
             builtin_programs,
-            sysvars,
+            Cow::Borrowed(sysvar_cache),
             log_collector,
             compute_budget,
             executors,
@@ -159,6 +159,7 @@ mod tests {
             keyed_account::keyed_account_at_index,
             message::Message,
             native_loader::{self, create_loadable_account_for_test},
+            pubkey::Pubkey,
             secp256k1_instruction::new_secp256k1_instruction,
             secp256k1_program,
         },
@@ -253,7 +254,7 @@ mod tests {
             )],
             Some(&accounts[0].0),
         ));
-
+        let sysvar_cache = SysvarCache::default();
         let result = MessageProcessor::process_message(
             builtin_programs,
             &message,
@@ -266,7 +267,7 @@ mod tests {
             Arc::new(FeatureSet::all_enabled()),
             ComputeBudget::new(),
             &mut ExecuteTimings::default(),
-            &[],
+            &sysvar_cache,
             Hash::default(),
             0,
             0,
@@ -296,7 +297,7 @@ mod tests {
             Arc::new(FeatureSet::all_enabled()),
             ComputeBudget::new(),
             &mut ExecuteTimings::default(),
-            &[],
+            &sysvar_cache,
             Hash::default(),
             0,
             0,
@@ -330,7 +331,7 @@ mod tests {
             Arc::new(FeatureSet::all_enabled()),
             ComputeBudget::new(),
             &mut ExecuteTimings::default(),
-            &[],
+            &sysvar_cache,
             Hash::default(),
             0,
             0,
@@ -463,6 +464,7 @@ mod tests {
             )],
             Some(&accounts[0].0),
         ));
+        let sysvar_cache = SysvarCache::default();
         let result = MessageProcessor::process_message(
             builtin_programs,
             &message,
@@ -475,7 +477,7 @@ mod tests {
             Arc::new(FeatureSet::all_enabled()),
             ComputeBudget::new(),
             &mut ExecuteTimings::default(),
-            &[],
+            &sysvar_cache,
             Hash::default(),
             0,
             0,
@@ -509,7 +511,7 @@ mod tests {
             Arc::new(FeatureSet::all_enabled()),
             ComputeBudget::new(),
             &mut ExecuteTimings::default(),
-            &[],
+            &sysvar_cache,
             Hash::default(),
             0,
             0,
@@ -540,7 +542,7 @@ mod tests {
             Arc::new(FeatureSet::all_enabled()),
             ComputeBudget::new(),
             &mut ExecuteTimings::default(),
-            &[],
+            &sysvar_cache,
             Hash::default(),
             0,
             0,
@@ -585,7 +587,7 @@ mod tests {
             ],
             None,
         ));
-
+        let sysvar_cache = SysvarCache::default();
         let result = MessageProcessor::process_message(
             builtin_programs,
             &message,
@@ -598,7 +600,7 @@ mod tests {
             Arc::new(FeatureSet::all_enabled()),
             ComputeBudget::new(),
             &mut ExecuteTimings::default(),
-            &[],
+            &sysvar_cache,
             Hash::default(),
             0,
             0,
